pub struct InsertionError<T, S: Storage<T>> {
pub item: T,
pub child_offset: usize,
pub child_id: S::Node,
}Expand description
Error returned when a direct insertion by key in the internal node failed.
Fields
item: TInserted key.
child_offset: usizeOffset of the child in which the key should be inserted instead.
child_id: S::NodeId of the child in which the key should be inserted instead.