pub type RightItem<T, S> = (Offset, T, Option<<S as Storage<T>>::Node>);Expand description
Type of the value returned by Node::pop_right.
It includes the offset of the popped item, the item itself and the index of the right child of the item if it is removed from an internal node.