Struct raw_btree::RawBTree

source ·
pub struct RawBTree<T, S: Storage<T> = BoxStorage> { /* private fields */ }

Implementations

source

impl<T, S: Storage<T>> RawBTree<T, S>

source

pub fn new() -> RawBTree<T, S>

Create a new empty B-tree.

source

pub fn is_empty(&self) -> bool

source

pub fn len(&self) -> usize

source

pub fn address_of<Q: ?Sized>( &self, cmp: impl Fn(&T, &Q) -> Ordering, key: &Q ) -> Result<Address<S::Node>, Option<Address<S::Node>>>

source

pub fn first_item_address(&self) -> Option<Address<S::Node>>

source

pub unsafe fn get_at(&self, addr: Address<S::Node>) -> Option<&T>

Return the item at the given address.

Safety

The address’s node must not have been deallocated.

source

pub unsafe fn get_mut_at(&mut self, addr: Address<S::Node>) -> Option<&mut T>

Returns a mutable reference to the item at the given address.

Safety

The address’s node must not have been deallocated.

source

pub fn get<Q: ?Sized>( &self, cmp: impl Fn(&T, &Q) -> Ordering, key: &Q ) -> Option<&T>

source

pub fn get_mut<Q: ?Sized>( &mut self, cmp: impl Fn(&T, &Q) -> Ordering, key: &Q ) -> Option<&mut T>

source

pub fn first(&self) -> Option<&T>

source

pub fn first_mut(&mut self) -> Option<&mut T>

source

pub fn last(&self) -> Option<&T>

source

pub fn last_mut(&mut self) -> Option<&mut T>

source

pub fn iter(&self) -> Iter<'_, T, S>

source

pub fn iter_mut(&mut self) -> IterMut<'_, T, S>

source

pub fn insert(&mut self, cmp: impl Fn(&T, &T) -> Ordering, item: T) -> Option<T>

source

pub fn remove<Q: ?Sized>( &mut self, cmp: impl Fn(&T, &Q) -> Ordering, key: &Q ) -> Option<T>

Remove the next item and return it.

source

pub fn visit_from_leaves(&self, f: impl FnMut(S::Node))

source

pub fn visit_from_leaves_mut(&mut self, f: impl FnMut(S::Node, &mut Node<T, S>))

source

pub fn forget(&mut self)

source

pub fn clear(&mut self)

source

pub fn validate(&self, cmp: impl Fn(&T, &T) -> Ordering)

source

pub fn validate_node( &self, cmp: &impl Fn(&T, &T) -> Ordering, id: S::Node, parent: Option<S::Node>, min: Option<&T>, max: Option<&T> ) -> usize

Validate the given node and returns the depth of the node.